module Store
  module Api
    class Store::ProductsController < ::Store::ApplicationController
      before_action :set_product, only: [:show, :update, :destroy]

      def index
        p = params.to_unsafe_h.except('controller', 'action', 'uuid', 'product', 'auth_token', 'format', 'sort', 'page')
        search=Search::ProductsSearch.new(p, user: @user).search

        @products = search.sorter(params[:sort])
        paginate @products, each_serializer: ProductSerializer
      end

      def show
        if @product
          render jsonapi: @product, serializer: ProductSerializer
        else
          render_json_error :not_found, :product_not_found
        end

      end

      def create
        if @user.auth_roles.to_a.include?('Developper') || @user.roles.to_a.include?('Administrator')
          @product = Product.new(product_params)
          @product.user_id = @user.id
          if @product.save
            render json: @product, status: :created #, location: @product
          else
            respond_with_errors(@product)
          end
        else

          render_json_error :not_found, :not_admin_or_manager
        end
      end

      def update
        if @product.reload && @product.writable_by?(@user)
          if @product.update(product_params)
            render json: @product
          else
            respond_with_errors(@product)
          end
        else
          render_json_error :unprocessable_entity, :not_writers
        end
      end

      def destroy
        if @product
          if current_user.has_role?("Administrator") || current_user.has_role?("Manager")
            if @product.destroy
              render body: nil, status: :no_content
            else
              render json: @product.errors, status: :unprocessable_entity
            end
          else
            render_json_error :not_found, :not_admin_or_manager
          end
        else
          render_json_error :not_found, :product_not_found
        end
      end

      def current_user
        @user
      end

      def set_product
        @product = Product.readable_by(current_user).find_by(id: params[:id])
      end

      private
      def product_params
        ActiveModelSerializers::Deserialization.jsonapi_parse!(params, only: [:title, :body, :price, :vat, :duration_count, :duration_unit, :writers, :readers] )
      end
    end
  end
end