module Store
  class Order
    include Mongoid::Document
    include Mongoid::Timestamps

    field :browser_ip, type: String
    field :currency, type: String, default: 'EUR'
    field :total_price, type: Float
    field :host, type: String # to generate licence for on host ex: https://paw.astrimmo.com

    field :payment_amount, type: Float, default: 0.0
    field :payment_method, type: String
    field :payment_status, type: String, default: 'not_processed'
    field :product_ids, type: Array, default: []
      # pending: The payments are pending. Payment might fail in this state. Check again to confirm whether the payments have been paid successfully.
      # authorized: The payments have been authorized.
      # partially_paid: The order have been partially paid.
      # paid: The payments have been paid.
      # partially_refunded: The payments have been partially refunded.
      # refunded: The payments have been refunded.
      # voided: The payments have been voided.
    field :cancel_reason, type: String

    belongs_to :user
    belongs_to :customer, class_name: 'Store::Customer'
    has_many :vouchers, class_name: 'Store::Voucher'

    before_save :calc_price

    after_save :gen_vouchers

    include ::AclProtected

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    def products
      Product.where(:id.in => self.product_ids)
    end

    def calc_price
      if self.product_ids_changed?
        self.total_price = self.products.sum(:price).to_f
        if self.payment_amount == self.total_price
          self.payment_status = 'paid'
        elsif self.payment_amount > 0
          self.payment_status = 'partially_paid'
        end
      end
    end

    def gen_vouchers
      if self.payment_status == 'paid' && self.vouchers.count == 0
        self.products.each do |product|
          Voucher.create(product_id: product.id, order_id: self.id)
        end
      end
    end

  end
end
