module Store
  class Product
    include Mongoid::Document
    include Mongoid::Timestamps

    include ::AclProtected
    include Concerns::Data
    include Slugable

    field :title, type: String
    field :body, type: String
    field :type, type: String, default: 'app' # used for app
    field :package, type: String # to generate voucher license exemple : 'emails'
    field :labels, type: String
    field :superfields, type: Hash
    field :picture_ids, type: Array, default: []
    field :values, type: Hash, default: {}
    # values contains number of seats, number of users or other product variations to generate license values
    field :duration_count, type: Integer, default: 3
    field :duration_unit, type: String, default: 'day'

    field :price, type: Float
    field :vat, type: Float, default: 20.0

    belongs_to :user

    validates_presence_of :price

    index({ title: 1 }, { background: 1 })

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    sluggify :title

    # Store::Product.create(duration_count: 1, duration_unit: 'year', package: 'contracts', title: '1 year contracts app')
    def gen_code_for(host)
      expiration = Date.today + self.duration_count.send("#{duration_unit}")
      values=self.values
      l=License.create(type: self.type, module: self.package, expire_at: expiration, env: Rails.env) #, active: true)
      l.token=SecureRandom.uuid
      values['app_host']=host
      l.values=values.stringify_keys
      l.save
      l.key=Base64.encode64(OpenSSL::HMAC.digest(OpenSSL::Digest.new('sha256'), l.token, "#{l.expire_at}#{l.values}")).strip
      l.save
      code=l.reload.to_code
      l.destroy
      code
    end
  end
end