# frozen_string_literal: true

module Store
  # Base serializer
  class ApplicationSerializer < ActiveModel::Serializer
    cache(
      key: proc { "#{object.cache_key}?read_level=#{read_level}" },
      expires_in: 2.hours
    )
    attributes :id, :readers, :writers, :read_level

    def id
      object&.id&.to_s
    end

    attributes :created_at, :updated_at

    def writable?
      return false unless scope

      object&.writable_by?(scope)
    end

    def writers
      writable? ? object&.writers : nil
    end

    def readable?
      return false unless scope

      try(:is_writable?) || object&.readable_by?(scope)
    end

    def readers
      readable? ? object&.readers : nil
    end

    def read_level
      if writable?
        'write'
      elsif readable?
        'read'
      else
        'guest'
      end
    end
  end
end
