require 'rails_helper'

module Store
  describe Api::Store::VouchersController, type: :api do
    routes {Store::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:admin) { create(:administrator) }

    let!(:responsible) { create(:user) }
    let!(:user) { create(:user) }

    let!(:voucher1) { create(:voucher, host: 'rec.plugandwork.fr', module: 'emails', type: 'app') }
    let!(:voucher2) { create(:voucher, host: 'rec.plugandwork.fr', module: 'files', type: 'app') }
    let!(:voucher3) { create(:voucher, host: 'rec.plugandwork.fr', module: 'todo', type: 'app') }

    let!(:peer) { create(:peer, active: true, host: 'rec.plugandwork.fr', token: "123") }

    describe 'GET /products' do
      context 'with valid peer token' do
        before :each do
          get 'index', token: peer.token
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty

          expect(json['data'].length).to eq(3)
          expect(json['data'][0]['attributes']['host']).to eq('rec.plugandwork.fr')
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(3)
        end
      end
    end

  end
end